# Programa en python
# para calibrar una cámara usando un patrón de círculos
# 
# Fraga 03.08.2021
import sys
import numpy as np
import numpy.linalg as linalg

n = len( sys.argv )
if n != 3 :
	print( "Args: elipse1.bin elipse2.bin" )
	sys.exit(1)

# Se leen las dos elipses de los archivos en binario
# en donde están almacenadas
E1 = np.fromfile( sys.argv[1] )
E1 = E1.reshape( (3,3) )

E2 = np.fromfile( sys.argv[2] )
E2 = E2.reshape( (3,3) )

# Como es una simulación podemos saber
# la raíces :
beta1 = E2[2,2]/E1[2,2]
print( "beta1 =", beta1 )

beta3 = beta1*(1.0/0.5)*(1.0/0.5)
print( "beta3 =", beta3 )

# Ponemos a el mismo factor de escala las dos elipses
HE1 = 1000.0 * ( E1 / E1[2,2] )
print( HE1 )
HE2 = 1000.0 * ( E2 / E2[2,2] )
print( HE2 )

# Invertimos las elipses:
Q1 = linalg.inv( HE1 )
# print( Q1inv )
Q2 = linalg.inv( HE2 )
# print( Q2inv )

B = HE1 @ Q2

vv, D = linalg.eig( B )
# Se podría resolver directamente la cúbica a partir
# de la matriz B para hallar la eigenvalores de B

# Los eigenvalores no están necesariamente ordenados.
# Hallamos la raíz repetida con el procedimiento del paper
print( "vv =", vv )

# La raíz repetida
b1 = vv[ np.argmin( vv ) ]

# La raíz única
b3 = vv[ np.argmax( vv ) ]
print( "b1,b3:", b1, b3 )


# Recuperamos H
# Para la raiz igual

# Para la tercera columna de H, usamos la raiz repetida
# Usamos la eigendescomposición de Delta (esta es simétrica)
Delta = b1*Q1 - Q2
vd2, V2 = linalg.eig(Delta)

H = np.eye( 3 )
H[:,2] = V2[:,0]*np.sqrt( vd2[0] )


# Para hallar las otras dos columnas de H, usamos la raíz única
Delta = b3*Q1 - Q2
vd3, V3 = linalg.eig(Delta)
print( "vd3:", vd3 )

H[:,0] = V3[:,0]*np.sqrt( vd3[0] )
H[:,1] = V3[:,1]*np.sqrt( vd3[1] )

print( "H:" )
print( H )

u0 = 300
v0 = 200
f2 = ( u0 * ( H[0,1] * H[2,0] + H[2,1]*H[0,0] - u0*H[2,1]*H[2,0] ) + v0*( H[1,1]*H[2,0] + 
     H[2,1]*H[1,0] - v0*H[2,1]*H[2,0] ) - H[0,1]*H[0,0] - H[1,1]*H[1,0] )/ (H[2,1]*H[2,0])

f2 = np.sqrt(f2)
print( f2 )
